<?php
/*
 * génère la liste des paquets winget à installer sur le poste.
 *
 * On se base sur le nom de la machine, c'est suffisant pour récup les infos nécessaires, et il n'y a rien de confidentiel
 *
 * - Par défaut on met à jour les paquets déjà installés, sauf si une version inférieure est spécifiée dans la définition xml du package.
 * - si une source winget est définie elle est utilisée pour les installations/mises à jour. Sinon on utilise la source MS officielle
 * - Si l'id est spécifié dans le XML, alors il sera installé par winget si il ne l'est pas déjà.
 * - NON, winget gère les dépendances Si un package avec un id est défini dans la liste des applications globales, mais n'est pas demandé pour le poste, il sera désinstallé par Winget
 * - si un package est présent dans la liste des applications à desinstaller on le retire
 *
 */
include "config.inc.php";
$config = get_config();
include ("wpkg_lib.php");
include ("wpkg_libsql.php");
include ("ldap.inc.php");

$action = $_POST['action'] ?? "";
$machine = $_POST['machine'] ?? "";
$list = $_POST["list"] ?? "";
if (empty($config['winget'])) {
    header("HTTP/1.1 400 Bad request");
    exit();
}

if (! empty($list) && ! empty($machine) && $action == "list") {
    $local_apps = json_decode($list, true);
    file_put_contents("/tmp/winget_local_apps.json", json_encode($local_apps, JSON_PRETTY_PRINT));
    $winget = [];
    /*
     * $liste_applications = liste_applications($config);
     * $xml = new DOMDocument();
     * $xml->formatOutput = true;
     * $xml->preserveWhiteSpace = false;
     * $xml->load($url_packages);
     * $packages = $xml->documentElement->getElementsByTagName('package');
     * $liste_all = [];
     * foreach ($packages as $package) {
     * if (array_key_exists(hash('md5', $package->getAttribute('id')), $liste_applications)) {
     * $variables = $package->getElementsByTagName("windows");
     * foreach ($variables as $variable) {
     * if ($variable->getAttribute('type') == "winget") {
     * if ($variable->getAttribute('version') != "") {
     * $app['Version'] = $variable->getAttribute('version');
     * }
     * if ($variable->getAttribute('source') != "") {
     * $app['Source'] = $variable->getAttribute('source');
     * } else {
     * $app['Source'] = "winget";
     * }
     * $app['Id'] = $variable->getAttribute('id');
     * $liste_all[] = $app;
     * }
     * }
     * }
     * }
     */

    $liste_applications = info_poste_applications($config, $machine);
    $xml = new DOMDocument();
    $xml->formatOutput = true;
    $xml->preserveWhiteSpace = false;
    $xml->load($url_packages);
    $packages = $xml->documentElement->getElementsByTagName('package');

    $liste = [];
    $liste_applications = array_column(array_column($liste_applications, 'info_app'), 'id_nom_app');
    foreach ($packages as $package) {
        if (is_int(array_search(strtolower($package->getAttribute('id')), array_map("strtolower", $liste_applications)))) {
            $variables = $package->getElementsByTagName("windows");
            foreach ($variables as $variable) {
                if ($variable->getAttribute('type') == "winget") {
                    if ($variable->getAttribute('version') != "") {
                        $app['Version'] = $variable->getAttribute('version');
                        // $p['Version'] = $variable->getAttribute('version');
                    }
                    if ($variable->getAttribute('source') != "") {
                        $app['Source'] = $variable->getAttribute('source');
                    } else {
                        $app['Source'] = "winget";
                    }
                    // ajout d'un argument d'installation
                    if ($variable->getAttribute('custom') != "") {
                        $app['Custom'] = $variable->getAttribute('custom');
                    }
                    // remplacement de tous les arguments d'installation
                    // par exemple pour office : "/configure C:/path/to/config/file.xml"
                    if ($variable->getAttribute('override') != "") {
                        $app['Override'] = $variable->getAttribute('override');
                    }
                    $app['Id'] = $variable->getAttribute('id');
                    // $p['Id'] = $variable->getAttribute('id');
                    $liste[] = $app;
                    // $pin[] = $p;
                }
            }
        }
    }
    file_put_contents("/tmp/winget_wpkg.json", json_encode($liste, JSON_PRETTY_PRINT));

    $filename = "/etc/sambaedu/applications/winget/add.json";
    if (file_exists($filename)) {
        $l_add = json_decode(file_get_contents($filename), true) ?? [];
    } else {
        $l_add = [];
    }
    $filename = "/usr/share/sambaedu/applications/winget/add.json";
    if (file_exists($filename)) {
        $add = json_decode(file_get_contents($filename), true); // ?? [];
    } else {
        $add = [];
    }
    foreach ($add as $a) {
        $k = array_search($a['Id'], array_column($l_add, "Id"));
        if ($k !== false) {
            unset($l_add[$k]);
        }
    }
    $add = array_merge($add, $l_add);
    file_put_contents("/tmp/winget_add.json", json_encode($add, JSON_PRETTY_PRINT));
    foreach ($add as $a) {
        $k = array_search($a['Id'], array_column($liste, "Id"));
        if ($k !== false) {
            unset($add[$k]);
        }
    }
    $liste = array_merge($add, $liste);
    file_put_contents("/tmp/winget_liste.json", json_encode($liste, JSON_PRETTY_PRINT));

    // A installer / upgrade
    foreach ($liste as $l) {
        $k = array_search($l['Id'], array_column($local_apps, "Id"));
        if ($k !== false) {
            // déjà installé, on vérifie la version
            if ($local_apps[$k]['IsUpdateAvailable']) {
                unset($l['AvailableVersion']);
                if (isset($l['Version'])) {
                    if (version_compare($l['Version'], $local_apps[$k]['InstalledVersion'], "gt")) {
                        foreach ($local_apps[$k]['AvailableVersions'] as $v) {
                            if (version_compare($l['Version'], $v, "gt")) {
                                break;
                            }
                            $l['AvailableVersion'] = $v;
                        }
                    }
                } else {
                    $l['AvailableVersion'] = $local_apps[$k]['AvailableVersions'][0];
                }
                $l['Version'] = $local_apps[$k]['InstalledVersion'];
                if (isset($l['AvailableVersion'])) {
                    $winget['upgrade'][] = $l;
                }
            }
        } else {
            $winget['install'][] = $l;
        }
    }

    // a retirer : attention winget gère auto les dépendances, donc on n'enlève pas les paquets installés non demandés !
    // essentiellement utile pour debloater Windows.
    // uninstall : pour le moment uniquement la liste ci-dessous
    $filename = "/etc/sambaedu/applications/winget/remove.json";
    if (file_exists($filename)) {
        $l_poubelle = json_decode(file_get_contents($filename), true) ?? [];
    } else {
        $l_poubelle = [];
    }
    $filename = "/usr/share/sambaedu/applications/winget/remove.json";
    if (file_exists($filename)) {
        $poubelle = json_decode(file_get_contents($filename), true); // ?? [];
    } else {
        $poubelle = [];
    }
    foreach ($poubelle as $p) {
        $k = array_search($p['Id'], array_column($l_poubelle, "Id"));
        if ($k !== false) {
            unset($l_poubelle[$k]);
        }
    }
    $poubelle = array_merge($poubelle, $l_poubelle);
    file_put_contents("/tmp/winget_poubelle.json", json_encode($poubelle, JSON_PRETTY_PRINT));
    foreach ($local_apps as $l) {
        $k = array_search($l['Id'], array_column($poubelle, "Id"));
        if ($k !== false) {
            $winget['uninstall'][] = $l;
        }
    }
    // @TODO remonter l'info vers wpkg !
    file_put_contents("/tmp/winget_out.json", json_encode($winget, JSON_PRETTY_PRINT));

    header('Content-type: text/json');
    echo json_encode($winget, JSON_PRETTY_PRINT);
} else {
    header("HTTP/1.1 400 Bad request");
}
?>